/*********************************************************************
 *
 * 	Fichero en el que se incluyen funciones varias tales como:
 *     config_pic() Fichero donde se incluyen funciones de configuracin del PIC.
 *     proceso_rx() Funcin utilizada para el proceso de Keep Alive, si este 
 *	   esta activo, cada X seg se encarga de enviar un mensaje
 *	   sobre el estado de las puertas.
 *	   ConfigPuertas() Establece el estado de las puertas, si fueran salidas o entradas.
 *	   Temporizador() Establece los distintos tiempos de muestreo disponibles, segun 
 *	   se indique.
 *
 *********************************************************************
 * FileName:        funciones.c
 *
 * Author  			Alejandro Daniel Pez Espinosa     
 ********************************************************************/
 
#include "funciones.h"
#include "TramaEthernet.h"

void ConfigPic()
{
	setup_adc_ports(NO_ANALOGS|VSS_VDD);
	setup_adc(ADC_CLOCK_DIV_2|ADC_TAD_MUL_0);
	setup_psp(PSP_DISABLED);
	setup_spi(SPI_SS_DISABLED);
	setup_spi2(SPI_SS_DISABLED);
	setup_wdt(WDT_OFF);
	setup_ccp1(CCP_OFF);
	setup_comparator(NC_NC_NC_NC);
	setup_vref(FALSE);
}

void ProcesoRx()
{
	if( keepalive )
	{
		if( count_keepalive > numKeepAlive )
		{
			enviar( SEND_IN , 1 );
			count_keepalive=0;
			
		}
	}	
}	


void CounterReset()
{
	contador[0]=0;
    contador[1]=0; 
}  

void AddCount(){
     if(contador[1]==255 && contador[0]==255)
     {
          CounterReset();
     }
     else if(contador[1]==255)
     {
          contador[1]=0;
          contador[0]++;
     }
     else
     {
          contador[1]++;
     }               
               
}

void ConfigPuertas(){
	// FF entradas, 00 salidas
	int i;
	
	// PINES Entradas Total 31
	set_tris_b(0xff);
	set_tris_c(0xe7); 
		/* C7, C6, C5, C2, C1, C0 	ENTRADAS
		 * C4, C3 					SALIDAS
		 * C7, C6, C5, C4, C3 son puertas reservadas con lo que se mantienen en este formato
		 * C0, C1, C2 puertas libres para su uso */
	set_tris_d(0xef);
		/* C7, C6, C5, C3, C2, C1, C0	ENTRADAS
		 * C4							SALIDA
		 * D0, D2, D3, D4 son puertas reservadas
		 * D1, D5, D6, D7 puertas libres para su uso */
	set_tris_f(0xff);
	set_tris_g(0xff);
	
	// PINES Salidas Total 30
	set_tris_a(0x00);
	set_tris_e(0x00);
	set_tris_h(0x00);
	set_tris_j(0x00);
	
	//Colocamos todas las salidas a 0
	for(i=0;i<8;i++){
		bit_clear(PORTE,i);
		bit_clear(PORTH,i);
		bit_clear(PORTJ,i);
	}
	for(i=0;i<6;i++)
		bit_clear(PORTA,i);
}	

void Temporizador(int x){
	switch(x){
		case 1:
			setup_timer_1(T1_INTERNAL|T1_DIV_BY_1);
			break;
		case 2:
			setup_timer_1(T1_INTERNAL|T1_DIV_BY_2);
			break;
		case 3:
			setup_timer_1(T1_INTERNAL|T1_DIV_BY_4);
			break;
		case 4:
			setup_timer_1(T1_INTERNAL|T1_DIV_BY_8);
			break;
	}			
}	
/* Configura el modo de funcionamiento del PIC, si retorna FALSE, es que el modo no ha sido 
correcto. En caso favorable retornara un TRUE */
int1 SeleccionModo()
{
	int8 iden_pic;
	int8 bps;
	int8 modo=rx_trama[2];	
	iden_pic=rx_trama[1];
	
	switch( modo )
	{
		case PortSerie:
						flag_red=FALSE;
						flag_serie=TRUE;
						/* Si es modo serie comprobamos que velocidad se quiere utilizar para la configuracion, por 
	 					* defecto y de modo incial se utiliza 9600bps */
						bps=rx_trama[3];
						switch( bps ){
							case 1:
								setup_uart(9600);
								break;
							case 2:
								setup_uart(14400);
								break;
							case 3:
								setup_uart(19200);
								break;
							case 4:
								setup_uart(38400);
								break;
							case 5:
								setup_uart(57600);
								break;
							case 6:
								setup_uart(115200);
								break;
							case 7:
								setup_uart(256000);
								break;
						}
						setup_timer_0(RTCC_INTERNAL|RTCC_DIV_64);	// Se establece el tiempo desbordamiento en 524ms aprox.
						break;

		case PortEthernet:
		
						flag_red=TRUE;
						flag_serie=FALSE;
						disable_interrupts(INT_RDA);	// Se desactiva ya que no sera necesaria la recepcion por
														// puerto serie
						/* Por defecto se establece DCHP activo, si se quiere desactivar y usar la IP 
						 * por defecto se desactiva.
						 */
						EthernetInit(); 				// Iniciamos el modo red

						if ( rx_trama[3] == 0 ) // Sino se estable ninguna IP
						{
							IPAddrInit();
						}
						else // Si se cambia la IP por defecto
						{
							//Direccion IP
							MY_IP_BYTE1=rx_trama[3];
							MY_IP_BYTE2=rx_trama[4];
							MY_IP_BYTE3=rx_trama[5];
							MY_IP_BYTE4=rx_trama[6];
							
							//Mascara de Subred
							MY_MASK_BYTE1=rx_trama[7];
							MY_MASK_BYTE2=rx_trama[8];
							MY_MASK_BYTE3=rx_trama[9];
							MY_MASK_BYTE4=rx_trama[10];
						}	

						break;
		default: 
			return TRUE;				
	}
				
	if( iden_pic!=0)
	{
		ID_PIC=iden_pic;
	}
	
	return FALSE;						
}	

