/*********************************************************************
 *
 *     Fichero til para el anlisis de la trama recibida, donde se 
 *		observar que tipo de trama recibimos. Contiene funciones para la 
 *		formacin y envi de trama ya sea mediante puerto serie o red. 
 *	  Tambin podemos encontrar un par de funciones con la que se realizan 
 *    el cambio de estado de las puertas lgicas.
 *
 *********************************************************************
 * FileName:        Trama.c
 *
 * Author  Alejandro Daniel Pez Espinosa     
 ********************************************************************/
 
#include "trama.h"
#include "funciones.h"
#include "TramaEthernet.h"

// Realiza el envio de la trama por el Puerto Serie o red
void enviar( char tipo_trama, char id_pc )
{
	int i;
	if( tipo_trama == SEND_IN ) AddCount();
	
	if( tipo_trama != ACK_ECHO )
	{
		for(i=0;i<NUM_PUERTAS;i++)
		{
			if(s_old_in[i]==1)
			{
				s_temp[i]=1;
			}	
			else
			{
				s_temp[i]=0;
			}	
		}
	}	
	
	formar_trama( tipo_trama , s_temp , id_pc);
	
	if ( flag_serie )
	{
		for( i=0 ; i<LONG_TRAMA ; i++ )
		{
			 putc(tx_trama[i]); 
			 rx_trama[i]=0;
		}
	}
	else if( flag_red )
	{
		EnviarMensajeUDP(tx_trama);
	}

	
}	

// Formar la trama para luego ser enviada al PC
void formar_trama(char tipo_trama, char datos[8], char id_pc)
{   
	 int i;
	 
	//emisor                                                    
	tx_trama[0]=ID_PIC;
	
	//receptor
	tx_trama[1]=id_pc; 
	    
	//longitud de los datos
	//trama[2]=(char) 47 + (sizeof(datos)/sizeof(char));
	tx_trama[2]=(char) 14;
	
	//tipo de trama
	//trama[3]=tipo_trama + 48;
	tx_trama[3]=tipo_trama;
		
	//datos                         
	for( i=0 ; i<8 ; i++)
	{
		tx_trama[4+i]=datos[i];  
	}	
	/* Se almacena los contadores de este modo ya que utilizamos el formato Big-Endian */
	tx_trama[12]=(char) contador[0];
	tx_trama[13]=(char) contador[1];     
} 


// Observa el campo modo de la trama y dependiendo de su modo realiza la accion solicitada
void mirar_trama()
{
	int1 entradas[64];
	char id_pc; // variabla utilizada para el id del PC
	int ok, modo, i;	
	modo=(int) rx_trama[3];
	//obtenemos el id_PC
	id_pc=rx_trama[1];

	/* Comprobamos que no nos envian una trama de configuracion inicial 
	 * y evitar problemas en el PIC */
	if( rx_trama[2] != PortSerie || rx_trama[2]!= PortEthernet)
	{
		switch( modo )
		{
			case ECHO:		
						enviar( ACK_ECHO , id_pc);
						break;
			case READ_IN:
						enviar( SEND_IN , id_pc);
						break;					
			case WRITE_IN: 
						for( i=0 ; i<NUM_PUERTAS ; i++) 
						{
							if(s_old_in[i]==1)
							{
								entradas[i]=1;
							}	
							else
							{
								entradas[i]=0;
							}	
						}
						ok=comparar_tramas(entradas);
			
						//Cambia con respecto a lo enviado	
				        if( ok == 0 ) 
				        {
					        write_output_all();
					        enviar( ACK_WRITE , id_pc );
				        }
				        else
				        {
							enviar( NO_ACK_WRITE , id_pc );
				        }
				        break;
				        
			case WRITE_IN_ONE:
						if( write_output_one()){
							enviar( ACK_WRITE_ONE , id_pc );
						}
						else
						{
							enviar( NO_ACK_WRITE_ONE , id_pc );
						}
						break;			
				        
			case POLLING:
					    if ( STATUS == STAND_BYE )
					    {
							STATUS=POLLING;
							th_in= rx_trama[5];
							Temporizador(rx_trama[4]);
							// Comprobamos si activamos el Keep Alive o no
							if( rx_trama [6] != 0 )
							{
								keepalive=TRUE;
								numKeepAlive=rx_trama[6];
							}	
							else
							{
								keepalive=FALSE;
							}	
							
						}
						break;
				
			case EVENT:
					    if( STATUS == STAND_BYE )
			    		{
			    			STATUS=EVENT;
			    			th_in=rx_trama[5];
			    			Temporizador(rx_trama[4]);
			    			if( rx_trama [6] != 0 )
			    			{
				    			keepalive=TRUE;
				    			numKeepAlive=rx_trama[6];
				    		}	
			    			else
			    			{
				    			keepalive=FALSE;
				    		}	
			    		}
			    		break;	
			    		  		
			case STOP:
						if( STATUS != STAND_BYE )
						{
							STATUS=STAND_BYE;
							th_in=rx_trama[5];
							Temporizador(rx_trama[4]);
							keepalive=FALSE;
						}
						break;	
	
		}	
	}				
}

/* 
funcion que retorna true o false, comprobando si las entradas 
estan como se han indicado o estan diferentes, ya que si se pide cambiar
por el mismo estado no realizamos los cambios ya que no serian necesarios
*/
int comparar_tramas(char entradas[(NUM_PUERTAS/8)])
{
     int i;
     for(i=0;i<(NUM_PUERTAS/8);i++)
     {
          if(entradas[i]!=rx_trama[4+i])return 0;     
     }          
     return 1;
}   

// Realiza el intercambio de los valores de todas las puertas logicas
void write_output_all()
{
	int8 puertas[32];
	int8 entradas[8];
	int16 i;
	int8 j;
	

	for( i=0 ; i<(NUM_PUERTAS/8) ; i++ )
	{
		entradas[i]=rx_trama[4+i];	
	}	
		
	for(i=0;i<8;i++){
		puertas[i]=bit_test(entradas[i/8],(i%8));
		puertas[i+8]=bit_test(entradas[(i+8)/8],((i+8)%8));
		puertas[i+16]=bit_test(entradas[(i+16)/8],((i+16)%8));
		puertas[i+24]=bit_test(entradas[(i+24)/8],((i+24)%8));
	}
	
	j=0;
	// Realiza cambio en puerta A 
	for( i=0 ; i<6 ; i++ )
	{
		if(puertas[j])
		{
			bit_set(PORTA,i);
		}	
		else
		{
			bit_clear(PORTA,i);
		}
		j++;			
	}

	j=8;

	// Realiza cambio en puerta E
	for( i=0 ; i<8 ; i++ )
	{
		if(puertas[j])
		{
			bit_set(PORTE,i);
		}	
		else
		{
			bit_clear(PORTE,i);
		}
		j++;			
	}

	j=12;
	for( i=0 ; i<8 ; i++ )
	{
		if(puertas[j])
		{
			bit_set(PORTH,i);
		}	
		else
		{
			bit_clear(PORTH,i);
		}
		j++;			
	}
	
	j=24;
	for( i=0 ; i<8 ; i++ )
	{
		if(puertas[j])
		{
			bit_set(PORTJ,i);
		}	
		else
		{
			bit_clear(PORTJ,i);
		}		
		j++;	
	}			
}	

//Realiza el cambio de estado de una puerta logica
int1 write_output_one()
{
	int8 num_puerta, estado;
	
	num_puerta=rx_trama[4];
	estado=rx_trama[5];
 
	// Realiza cambio en puerta A 
	if( num_puerta >=0 && num_puerta <6 )
	{
		if(!bit_test(PORTA,num_puerta) && estado==1 )
		{
			bit_set(PORTA,num_puerta);
			return TRUE;
		}	
		else if( bit_test(PORTA, num_puerta) && estado==0 )
		{
			bit_clear(PORTA,num_puerta);
			return TRUE;
		}	
		
	}		
		
	// Realiza cambio en puerta E
	if( num_puerta >7 && num_puerta <16 )
	{
		num_puerta=16-num_puerta;
		num_puerta=8-num_puerta;
		if(!bit_test(PORTE,num_puerta) && estado == 1 )
		{
			bit_set(PORTE,num_puerta);
			return TRUE;
		}	
		else if( bit_test(PORTE,num_puerta) && estado == 0 )
		{
			bit_clear(PORTE,num_puerta);
			return TRUE;
		}	
	}		

	// Realiza cambio en puerta H
	if( num_puerta >15 && num_puerta <24 )
	{
		num_puerta=24-num_puerta;
		num_puerta=8-num_puerta;
		if(!bit_test(PORTH,num_puerta) && estado == 1)
		{
			bit_set(PORTH,num_puerta);
			return TRUE;
		}	
		else if( bit_test(PORTH, num_puerta) && estado == 0)
		{
			bit_clear(PORTH,num_puerta);
			return TRUE;
		}	
	}		

	// Realiza cambio en puerta J
	if( num_puerta >23 && num_puerta <32 )
	{
		num_puerta=32-num_puerta;
		num_puerta=8-num_puerta;
		if(!bit_test(PORTJ,num_puerta) && estado == 1)
		{
			bit_set(PORTJ,num_puerta);
			return TRUE;
		}	
		else if( bit_test(PORTJ, num_puerta) && estado == 0)
		{
			bit_clear(PORTJ,num_puerta);
			return TRUE;
		}
	}
	
	return FALSE;		
	
}	
